<?php

namespace WeDevs\DokanPro\Modules\OrderMinMax\Frontend;

defined( 'ABSPATH' ) || exit;

/**
 * Segregates cart data for specific vendor
 *
 * @since 3.12.0
 */
class VendorCart {
	/**
	 * Product organized by vendor_id
	 *
	 * @since 3.12.0
	 *
	 * @var array
	 */
	protected array $product = array();

	/**
	 * Total amount organized by cart total amount by vendor_id
	 *
	 * @dince 3.12.0
	 *
	 * @var array
	 */
	protected array $total_amount = array();

	/**
	 * Initializes hooks
	 *
	 * @since 3.12.0
	 */
	public function __construct() {
		add_action( 'woocommerce_cart_loaded_from_session', array( $this, 'parse_cart' ) );
	}

	/**
	 * Parse cart by vendor
	 *
	 * @since 3.12.0
	 *
	 * @return void
	 */
	public function parse_cart(): void {
        if ( ! WC()->cart || WC()->cart->is_empty() ) { // If no item is on cart, return
			return;
		}
		foreach ( WC()->cart->get_cart() as $cart_item ) {
			$cart_product_id = ! empty( $cart_item['variation_id'] ) ? $cart_item['variation_id'] : $cart_item['product_id'];
            $cart_line_total = $cart_item['line_total'] ?? $cart_item['data']->get_price() * $cart_item['quantity'];

			$vendor    = dokan_get_vendor_by_product( $cart_product_id );
			$vendor_id = $vendor->get_id();

			if ( ! isset( $this->total_amount[ $vendor_id ] ) ) {
				$this->total_amount[ $vendor_id ] = 0.00;
			}

            if ( ! isset( $this->product[ $vendor_id ][ $cart_product_id ] ) ) {
                $this->product[ $vendor_id ][ $cart_product_id ] = array();
            }

			$this->product[ $vendor_id ][ $cart_product_id ][] = $cart_item;
			$this->total_amount[ $vendor_id ]                 += $cart_line_total;
		}
	}

	/**
	 * Fetches total amount of vendor in a customers cart
	 *
	 * @since 3.12.0
	 *
	 * @param int $vendor_id Vendor ID to fetch total amount
	 *
	 * @return float
	 */
	public function get_cart_total_by_vendor( int $vendor_id ): float {
		return $this->total_amount[ $vendor_id ] ?? 0.00;
	}

	/**
	 * Fetches quantity of a product already in cart of a specific vendor
	 *
	 * @since 3.12.0
	 *
	 * @param int $product_id Product ID to fetch quantity
	 *
	 * @return int
	 */
	public function get_product_quantity( int $product_id ): int {
		$vendor_id = dokan_get_vendor_by_product( $product_id, true );

        // Sum up all the quantities of the product
        $quantity = 0;
        if ( ! isset( $this->product[ $vendor_id ][ $product_id ] ) ) {
            return $quantity;
        }

        foreach ( $this->product[ $vendor_id ][ $product_id ] as $product ) {
            $quantity += $product['quantity'];
        }
		return $quantity;
	}

	/**
	 * Returns list of vendors available in the cart
	 *
	 * @since 3.12.0
	 *
	 * @return array
	 */
	public function get_vendor_id_list(): array {
		return array_keys( $this->total_amount );
	}

	/**
	 * Returns list of products available in the cart
	 *
	 * @since 3.12.0
	 *
	 * @return array
	 */
	public function get_product_id_list(): array {
		$vendor_id_list = $this->get_vendor_id_list();
		$product_ids    = array();
		foreach ( $vendor_id_list as $vendor_id ) {
			$product_ids = array( ...array_keys( $this->product[ $vendor_id ] ), ...$product_ids );
		}
		return $product_ids;
	}

	/**
	 * Gets cart grouped by vendor
	 *
	 * @since 3.12.0
	 *
	 * @return array
	 */
	public function get_cart_grouped_by_vendor(): array {
		return $this->product;
	}

	/**
	 * Gets a specific vendor's cart product list
	 *
	 * @since 3.12.0
	 *
	 * @param int $vendor_id Vendor ID to fetch products
	 *
	 * @return array
	 */
	public function get_cart_products_by_vendor( int $vendor_id ): array {
		return $this->product[ $vendor_id ] ?? array();
	}
}
